package com.autentia.hibernate;

import java.util.Iterator;
import java.util.List;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.commons.logging.LogFactory;
import org.apache.commons.logging.Log;
import org.apache.log4j.PropertyConfigurator;
import org.hibernate.Session;
import org.hibernate.Transaction;

/**
 * Unit test for simple App.
 */
public class AppTest extends TestCase {
	
	private static Log log = LogFactory.getLog(AppTest.class);

	/**
	 * Create the test case
	 * 
	 * @param testName name of the test case
	 */
	public AppTest(String testName) {
		super(testName);
	}

	/**
	 * @return the suite of tests being tested
	 */
	public static Test suite() {
		return new TestSuite(AppTest.class);
	}

	public void testSave() {

		log.debug("--------------------- Test Save ------------------------------");
		
		com.autentia.hibernate.Test test = new com.autentia.hibernate.Test();
		String cadenaLarga = new String();
		
		for (int i = 0; i < 400; i++) {
			cadenaLarga = cadenaLarga.concat("cadenalarga ");
		}
		log.debug("Metemos una cadena de " + cadenaLarga.length() + " caracteres.");
		
		test.setId(0);
		test.setText(cadenaLarga);
		
		// MySQL (hibernate.cfg.xml por defecto)
		
		Session mysqlSession = HibernateUtil.currentSession();
		Transaction tx = mysqlSession.beginTransaction();
		mysqlSession.save(test);
		tx.commit();
		HibernateUtil.closeSession();

		// Oracle
		
		HibernateUtil.addConfigFile("oracle.hibernate.cfg.xml", "oracle");
		Session oracleSession = HibernateUtil.currentSession("oracle");
		Transaction txOracle = oracleSession.beginTransaction();
		oracleSession.save(test);
		txOracle.commit();
		HibernateUtil.closeSession("oracle");

		// SQLServer
		
		HibernateUtil.addConfigFile("sqlserver.hibernate.cfg.xml", "sqlserver");
		Session sqlServerSession = HibernateUtil.currentSession("sqlserver");
		Transaction txSqlServer = sqlServerSession.beginTransaction();
		sqlServerSession.save(test);
		txSqlServer.commit();
		HibernateUtil.closeSession("sqlserver");

		test = null;
		cadenaLarga = null;
		
		log.debug("---------------------- Test save OK --------------------------");
	}

	public void testRead() {

		log.debug("--------------------- TestRead --------------------------------");
		
		List lResult = null;
		StringBuffer query = new StringBuffer("FROM Test");
		
		// MySQL (hibernate.cfg.xml por defecto)

		Session mysqlSession = HibernateUtil.currentSession();
		Transaction tx = mysqlSession.beginTransaction();
		lResult = mysqlSession.createQuery(query.toString()).list();
		tx.commit();
		HibernateUtil.closeSession();
		printResult("Mysql",lResult);
		lResult.clear();

		// Oracle

		HibernateUtil.addConfigFile("oracle.hibernate.cfg.xml", "oracle");
		Session oracleSession = HibernateUtil.currentSession("oracle");
		Transaction txOracle = oracleSession.beginTransaction();
		lResult = oracleSession.createQuery(query.toString()).list();
		txOracle.commit();
		HibernateUtil.closeSession("oracle");
		printResult("Oracle",lResult);
		lResult.clear();
		
		// SQLServer

		HibernateUtil.addConfigFile("sqlserver.hibernate.cfg.xml", "sqlserver");
		Session sqlServerSession = HibernateUtil.currentSession("sqlserver");
		Transaction txSqlServer = sqlServerSession.beginTransaction();
		lResult = sqlServerSession.createQuery(query.toString()).list();
		txSqlServer.commit();
		HibernateUtil.closeSession("sqlserver");
		printResult("SQLServer",lResult);
		lResult.clear();
		
		lResult = null;
		query = null;

		log.debug("------------------------ Test read OK ---------------------------");

	}
	
	
	public void printResult(String name, List result){
		log.debug("---------" + name + "----------");
		Iterator it = result.iterator();
		while (it.hasNext()){
			com.autentia.hibernate.Test t = (com.autentia.hibernate.Test)it.next();
			try {
				log.debug("Id: " + t.getId() + " Text: " + t.getText() + "...");
			} catch (Exception e) {
				// TODO Auto-generated catch block
				log.debug(e.getMessage());
			}
		}
		it = null;
	}

	public static void main(String[] args) {
		
		PropertyConfigurator.configure("log4j.properties");
		
		AppTest ap = new AppTest("");
		ap.testSave();
		ap.testRead();
		ap = null;
	}
}
